CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
174 lines
6.1 KiB
C++
174 lines
6.1 KiB
C++
/*
|
|
* Copyright (c) Advanced Micro Devices, Inc., or its affiliates.
|
|
*
|
|
* SPDX-License-Identifier: MIT
|
|
*/
|
|
|
|
/**
|
|
* @addtogroup hipMemRetainAllocationHandle hipMemRetainAllocationHandle
|
|
* @{
|
|
* @ingroup VirtualMemoryManagementTest
|
|
* `hipError_t hipMemRetainAllocationHandle(hipMemGenericAllocationHandle_t* handle,
|
|
* void* addr)` -
|
|
* Returns the allocation handle of the backing memory allocation given the address.
|
|
*/
|
|
|
|
#include <hip_test_kernels.hh>
|
|
#include <hip_test_common.hh>
|
|
|
|
#include "hip_vmm_common.hh"
|
|
|
|
#define DATA_SIZE (1 << 13)
|
|
|
|
/**
|
|
* Test Description
|
|
* ------------------------
|
|
* - Create a VM mapped to physical memory. Input addr to
|
|
* hipMemRetainAllocationHandle and validate the handle.
|
|
* ------------------------
|
|
* - unit/virtualMemoryManagement/hipMemRetainAllocationHandle.cc
|
|
* Test requirements
|
|
* ------------------------
|
|
* - HIP_VERSION >= 6.1
|
|
*/
|
|
HIP_TEST_CASE(Unit_hipMemRetainAllocationHandle_SetGet) {
|
|
HIP_CHECK(hipFree(0));
|
|
size_t granularity = 0;
|
|
constexpr int N = DATA_SIZE;
|
|
size_t buffer_size = N * sizeof(int);
|
|
int deviceId = 0;
|
|
hipDevice_t device;
|
|
HIP_CHECK(hipDeviceGet(&device, deviceId));
|
|
checkVMMSupported(device);
|
|
hipMemAllocationProp prop{};
|
|
prop.type = hipMemAllocationTypePinned;
|
|
prop.location.type = hipMemLocationTypeDevice;
|
|
prop.location.id = device; // Current Devices
|
|
HIP_CHECK(
|
|
hipMemGetAllocationGranularity(&granularity, &prop, hipMemAllocationGranularityMinimum));
|
|
REQUIRE(granularity > 0);
|
|
size_t size_mem = ((granularity + buffer_size - 1) / granularity) * granularity;
|
|
hipMemGenericAllocationHandle_t handle;
|
|
void* ptrA;
|
|
// Allocate physical memory
|
|
HIP_CHECK(hipMemCreate(&handle, size_mem, &prop, 0));
|
|
// Allocate virtual address range
|
|
HIP_CHECK(hipMemAddressReserve(reinterpret_cast<void**>(&ptrA), size_mem, 0, 0, 0));
|
|
HIP_CHECK(hipMemMap((void*)ptrA, size_mem, 0, handle, 0));
|
|
// Test hipMemRetainAllocationHandle
|
|
hipMemGenericAllocationHandle_t gethandle;
|
|
// Check beginning of VMM ptr
|
|
HIP_CHECK(hipMemRetainAllocationHandle(&gethandle, reinterpret_cast<void*>(ptrA)));
|
|
REQUIRE(gethandle == handle);
|
|
HIP_CHECK(hipMemRelease(gethandle));
|
|
HIP_CHECK(hipMemRelease(handle));
|
|
HIP_CHECK(hipMemUnmap(ptrA, size_mem));
|
|
HIP_CHECK(hipMemAddressFree(ptrA, size_mem));
|
|
}
|
|
|
|
/**
|
|
* Test Description
|
|
* ------------------------
|
|
* - Negative Tests
|
|
* ------------------------
|
|
* - unit/virtualMemoryManagement/hipMemRetainAllocationHandle.cc
|
|
* Test requirements
|
|
* ------------------------
|
|
* - HIP_VERSION >= 6.1
|
|
*/
|
|
HIP_TEST_CASE(Unit_hipMemRetainAllocationHandle_NegTst) {
|
|
HIP_CHECK(hipFree(0));
|
|
size_t granularity = 0;
|
|
constexpr int N = DATA_SIZE;
|
|
size_t buffer_size = N * sizeof(int);
|
|
int deviceId = 0;
|
|
hipDevice_t device;
|
|
HIP_CHECK(hipDeviceGet(&device, deviceId));
|
|
checkVMMSupported(device);
|
|
hipMemAllocationProp prop{};
|
|
prop.type = hipMemAllocationTypePinned;
|
|
prop.location.type = hipMemLocationTypeDevice;
|
|
prop.location.id = device; // Current Devices
|
|
HIP_CHECK(
|
|
hipMemGetAllocationGranularity(&granularity, &prop, hipMemAllocationGranularityMinimum));
|
|
REQUIRE(granularity > 0);
|
|
size_t size_mem = ((granularity + buffer_size - 1) / granularity) * granularity;
|
|
hipMemGenericAllocationHandle_t handle;
|
|
void* ptrA;
|
|
// Allocate physical memory
|
|
HIP_CHECK(hipMemCreate(&handle, size_mem, &prop, 0));
|
|
// Allocate virtual address range
|
|
HIP_CHECK(hipMemAddressReserve(&ptrA, size_mem, 0, 0, 0));
|
|
HIP_CHECK(hipMemMap(ptrA, size_mem, 0, handle, 0));
|
|
// Test hipMemRetainAllocationHandle
|
|
hipMemGenericAllocationHandle_t gethandle;
|
|
SECTION("nullptr handle") {
|
|
REQUIRE(hipMemRetainAllocationHandle(nullptr, reinterpret_cast<void*>(ptrA)) ==
|
|
hipErrorInvalidValue);
|
|
}
|
|
SECTION("nullptr Vmm ptr") {
|
|
REQUIRE(hipMemRetainAllocationHandle(&gethandle, nullptr) == hipErrorInvalidValue);
|
|
}
|
|
SECTION("not mapped address") {
|
|
void* ptrB;
|
|
HIP_CHECK(hipMemAddressReserve(&ptrB, size_mem, 0, 0, 0));
|
|
REQUIRE(hipMemRetainAllocationHandle(&gethandle, reinterpret_cast<void*>(ptrB)) ==
|
|
hipErrorInvalidValue);
|
|
HIP_CHECK(hipMemAddressFree(ptrB, size_mem));
|
|
}
|
|
HIP_CHECK(hipMemRelease(handle));
|
|
HIP_CHECK(hipMemUnmap(ptrA, size_mem));
|
|
SECTION("unmapped address") {
|
|
REQUIRE(hipMemRetainAllocationHandle(&gethandle, reinterpret_cast<void*>(ptrA)) ==
|
|
hipErrorInvalidValue);
|
|
}
|
|
HIP_CHECK(hipMemAddressFree(ptrA, size_mem));
|
|
}
|
|
|
|
HIP_TEST_CASE(Unit_hipMemRetainAllocationHandle_Capture) {
|
|
CTX_CREATE();
|
|
size_t granularity = 0;
|
|
size_t buffer_size = DATA_SIZE * sizeof(int);
|
|
int device_id = 0;
|
|
hipDevice_t device;
|
|
HIP_CHECK(hipDeviceGet(&device, device_id));
|
|
checkVMMSupported(device);
|
|
|
|
hipMemAllocationProp allocation_prop{};
|
|
allocation_prop.type = hipMemAllocationTypePinned;
|
|
allocation_prop.location.type = hipMemLocationTypeDevice;
|
|
allocation_prop.location.id = device;
|
|
|
|
HIP_CHECK(hipMemGetAllocationGranularity(&granularity, &allocation_prop,
|
|
hipMemAllocationGranularityMinimum));
|
|
REQUIRE(granularity > 0);
|
|
|
|
size_t allocation_size = ((granularity + buffer_size - 1) / granularity) * granularity;
|
|
hipMemGenericAllocationHandle_t allocation_handle;
|
|
void* device_ptr = nullptr;
|
|
HIP_CHECK(hipMemCreate(&allocation_handle, allocation_size, &allocation_prop, 0));
|
|
HIP_CHECK(hipMemAddressReserve(&device_ptr, allocation_size, 0, 0, 0));
|
|
HIP_CHECK(hipMemMap(device_ptr, allocation_size, 0, allocation_handle, 0));
|
|
|
|
hipMemGenericAllocationHandle_t retained_handle;
|
|
hipStream_t stream = nullptr;
|
|
HIP_CHECK(hipStreamCreate(&stream));
|
|
|
|
GENERATE_CAPTURE();
|
|
BEGIN_CAPTURE(stream);
|
|
HIP_CHECK(hipMemRetainAllocationHandle(&retained_handle, reinterpret_cast<void*>(device_ptr)));
|
|
END_CAPTURE(stream);
|
|
|
|
HIP_CHECK(hipStreamDestroy(stream));
|
|
HIP_CHECK(hipMemRelease(retained_handle));
|
|
HIP_CHECK(hipMemRelease(allocation_handle));
|
|
HIP_CHECK(hipMemUnmap(device_ptr, allocation_size));
|
|
HIP_CHECK(hipMemAddressFree(device_ptr, allocation_size));
|
|
CTX_DESTROY();
|
|
}
|
|
|
|
/**
|
|
* End doxygen group VirtualMemoryManagementTest.
|
|
* @}
|
|
*/
|